/*
** Copyright 2017-2019 Double Precision, Inc.
** See COPYING for distribution information.
*/

#include "libcxxw_config.h"
#include <x/exception.H>
#include <x/options.H>
#include "x/w/impl/richtext/richtextstring.H"
#include "x/w/impl/richtext/richtextmeta.H"
#include "x/w/text_param.H"
#include "x/w/text_param_literals.H"
#include "x/w/theme_text.H"
#include "screen.H"
#include "main_window.H"
#include "main_window_handler.H"
#include "x/w/impl/background_color.H"

using namespace LIBCXX_NAMESPACE::w;

void testtextparam(const main_window &mw)
{
	text_param param1{"foo", U"bar", rgb(0,0,0),
			theme_font{"sans_serif"},
			"0%"_color, "bar"};

	text_param param2(rgb(0,0,0), rgb(0,0,0),
			  "foo",
			  "sans_serif"_theme_font,
			  "0%"_color,
			  "foobaz");

	auto string=
		mw->impl->handler
		->create_richtextstring( {mw->get_screen()
					->impl
					->create_background_color("0%"),
					mw->impl->handler
					->create_current_fontcollection
					  (theme_font{"serif"})},
			param2);

	const auto &m=string.get_meta();
	const auto &s=string.get_string();

	if (m.size() != 2 ||
	    m.at(0).first != 0 ||
	    m.at(1).first != 3 ||
	    s.size() != 9 ||
	    m.at(0).second.bg_color.null() ||
	    !m.at(1).second.bg_color.null())
		throw EXCEPTION("Sumthin's wrong.");
}

void testrgb()
{
	std::string s=maroon;

	std::cout << s << std::endl;
	rgb color{s};

	if (color != maroon)
	{
		throw EXCEPTION("wrong.");
	}

	color=rgb{"b=80"};

	if (color != rgb{"r=0000, g=0000, b=8080, a=ffff"})
	{
		throw EXCEPTION("wrong.");
	}
}

void testthemetext()
{
	text_param t{ theme_text{"${font:some_theme_font}${color:white}"
				 "${decoration:underline}"
				 "O$$${decoration:none}k"}};


	if (t.string != U"O$k" ||
	    t.colors !=
	    std::unordered_map<size_t, text_color_arg>
	    {{0, text_color_arg{rgb{"r=ffff, g=ffff, b=ffff, a=ffff"}}}} ||
	    t.fonts != std::unordered_map<size_t, font_arg>
	    {{0, theme_font{"some_theme_font"}}} ||
	    t.decorations != std::unordered_map<size_t, text_decoration>{
		    {0, text_decoration::underline},
		    {2, text_decoration::none},
	    })
		throw EXCEPTION("theme_text parsing failed");
}

int main(int argc, char **argv)
{
	try {
		LIBCXX_NAMESPACE::locale::base::environment()->global();

		auto options=LIBCXX_NAMESPACE::option::list::create();

		options->addDefaultOptions();

		auto parser=LIBCXX_NAMESPACE::option::parser::create();

		parser->setOptions(options);

		int flag=parser->parseArgv(argc, argv);

		if (flag == 0)
			flag=parser->validate();

		if (flag == LIBCXX_NAMESPACE::option::parser::base::err_builtin)
			exit(0);

		auto mw=main_window::create([&]
					    (const auto &ignore)
					    {
					    });

		testtextparam(mw);
		testrgb();
		testthemetext();
	} catch (const LIBCXX_NAMESPACE::exception &e)
	{
		e->caught();
		exit(1);
	}
	return 0;
}
